diff --git a/i386/boot2/boot.c b/i386/boot2/boot.c
index 2f772fd..1496e80 100644
--- a/i386/boot2/boot.c
+++ b/i386/boot2/boot.c
@@ -91,8 +91,6 @@ int		bvCount = 0, gDeviceCount = 0;
 long		gBootMode; /* defaults to 0 == kBootModeNormal */
 BVRef		bvr, menuBVR, bvChain;
 
-static bool				checkOSVersion(const char * version);
-static void				getOSVersion();
 static unsigned long	Adler32(unsigned char *buffer, long length);
 //static void			selectBiosDevice(void);
 
@@ -209,7 +207,7 @@ static int ExecKernel(void *binary)
 	finalizeBootStruct();
 
 	// Jump to kernel's entry point. There's no going back now.
-	if ((checkOSVersion("10.7")) || (checkOSVersion("10.8")) || (checkOSVersion("10.9")))
+	if ((checkOSVersion("10.7")) || (checkOSVersion("10.8")) || (checkOSVersion("10.9")) || (checkOSVersion("10.10")))
 	{
 
 		// Notify modules that the kernel is about to be started
@@ -254,7 +252,7 @@ long LoadKernelCache(const char* cacheFile, void **binary)
 		verbose("Specified kernel cache file path = %s\n", cacheFile);
 	} else {
 		// Lion, Mountain Lion and Mavericks prelink kernel cache file
-		if ((checkOSVersion("10.7")) || (checkOSVersion("10.8")) || (checkOSVersion("10.9")))
+		if ((checkOSVersion("10.7")) || (checkOSVersion("10.8")) || (checkOSVersion("10.9")) || (checkOSVersion("10.10")))
 		{
 			snprintf(kernelCacheFile, sizeof(kernelCacheFile), "%skernelcache", kDefaultCachePathSnow);
 			verbose("10.7, 10.8 & 10.9 kernel cache file path = %s\n", kernelCacheFile);		}
@@ -551,9 +549,6 @@ void common_boot(int biosdev)
 			updateVRAM();
 		}
 
-		// Find out which version mac os we're booting.
-		getOSVersion();
-
 		if (platformCPUFeature(CPU_FEATURE_EM64T)) {
 			archCpuType = CPU_TYPE_X86_64;
 		} else {
@@ -671,7 +666,11 @@ void common_boot(int biosdev)
 			// bootFile must start with a / if it not start with a device name
 			if (!bootFileWithDevice && (bootInfo->bootFile)[0] != '/')
 			{
-				snprintf(bootFile, sizeof(bootFile), "/%s", bootInfo->bootFile); // append a leading /
+				if (checkOSVersion("10.10")) {
+					snprintf(bootFile, sizeof(bootFile), kDefaultKernelPathYosemite"%s", bootInfo->bootFile); // for Yosemite
+				} else {
+					snprintf(bootFile, sizeof(bootFile), kDefaultKernelPath"%s", bootInfo->bootFile); // append a leading /
+				}
 			} else {
 				strlcpy(bootFile, bootInfo->bootFile, sizeof(bootFile));
 			}
@@ -767,13 +766,14 @@ static void selectBiosDevice(void)
 
 bool checkOSVersion(const char * version)
 {
-	return ((gMacOSVersion[0] == version[0]) && (gMacOSVersion[1] == version[1])
-			&& (gMacOSVersion[2] == version[2]) && (gMacOSVersion[3] == version[3]));
-}
-
-static void getOSVersion()
-{
-	strncpy(gMacOSVersion, gBootVolume->OSVersion, sizeof(gMacOSVersion));
+    if ( (sizeof(version) > 4) && ('.' != version[4]) && ('\0' != version[4])) {  //10.10\0
+        return ((gMacOSVersion[0] == version[0]) && (gMacOSVersion[1] == version[1])
+                && (gMacOSVersion[2] == version[2]) && (gMacOSVersion[3] == version[3]))
+                && (gMacOSVersion[4] == version[4]);
+    } else {
+        return ((gMacOSVersion[0] == version[0]) && (gMacOSVersion[1] == version[1])
+                && (gMacOSVersion[2] == version[2]) && (gMacOSVersion[3] == version[3]));
+    }
 }
 
 #define BASE 65521L /* largest prime smaller than 65536 */
diff --git a/i386/boot2/boot.h b/i386/boot2/boot.h
index 00c6f9d..5860529 100644
--- a/i386/boot2/boot.h
+++ b/i386/boot2/boot.h
@@ -32,6 +32,10 @@
  * Paths used by chameleon
  */
 
+//kernel path
+#define kDefaultKernelPath		"/"
+#define kDefaultKernelPathYosemite	"/System/Library/Kernels/"
+
 //kernel cache
 #define kDefaultCachePathLeo "/System/Library/Caches/com.apple.kernelcaches/"
 #define kDefaultCachePathSnow "/System/Library/Caches/com.apple.kext.caches/Startup/"
@@ -70,6 +74,7 @@
 #define kScanSingleDriveKey	"Scan Single Drive"
 #define kInstantMenuKey		"Instant Menu"
 #define kDefaultKernel		"mach_kernel"
+#define kDefaultKernelYosemite	"kernel"
 #define kGUIKey			"GUI"
 #define kBootBannerKey		"Boot Banner"
 #define kShowInfoKey		"ShowInfo"		// gui.c
@@ -314,4 +319,6 @@ void HibernateBoot(char *boot_device);
 /* bmdecompress.c */
 void * DecompressData(void *srcbase, int *dw, int *dh, int *bytesPerPixel);
 
+bool checkOSVersion(const char * version);
+
 #endif /* !__BOOT2_BOOT_H */
diff --git a/i386/boot2/drivers.c b/i386/boot2/drivers.c
index 709e93e..576b695 100644
--- a/i386/boot2/drivers.c
+++ b/i386/boot2/drivers.c
@@ -241,7 +241,7 @@ long LoadDrivers( char * dirSpec )
 					return -1;
 				}
 			} else {
-				if (gMacOSVersion[3] == '9') {
+				if ((checkOSVersion("10.9")) || (checkOSVersion("10.10"))) {
 					strlcpy(gExtensionsSpec, dirSpec, 4087); /* 4096 - sizeof("Library/") */
 					strcat(gExtensionsSpec, "Library/");
 					FileLoadDrivers(gExtensionsSpec, 0);
diff --git a/i386/boot2/gui.c b/i386/boot2/gui.c
index be13191..d854f4e 100644
--- a/i386/boot2/gui.c
+++ b/i386/boot2/gui.c
@@ -44,6 +44,8 @@ enum {
 	iDeviceGeneric_o,
 	iDeviceHFS,
 	iDeviceHFS_o,
+	iDeviceHFS_yos,
+	iDeviceHFS_yos_o,
 	iDeviceHFS_mav,
 	iDeviceHFS_mav_o,
 	iDeviceHFS_ML,
@@ -59,6 +61,8 @@ enum {
 
 	iDeviceHFSRAID,
 	iDeviceHFSRAID_o,
+	iDeviceHFSRAID_yos,
+	iDeviceHFSRAID_yos_o,
 	iDeviceHFSRAID_mav,
 	iDeviceHFSRAID_mav_o,
 	iDeviceHFSRAID_ML,
@@ -126,6 +130,8 @@ image_t images[] = {
 	{.name = "device_generic_o",            .image = NULL},
 	{.name = "device_hfsplus",              .image = NULL},
 	{.name = "device_hfsplus_o",            .image = NULL},
+	{.name = "device_hfsplus_yos",          .image = NULL},
+	{.name = "device_hfsplus_yos_o",        .image = NULL},
 	{.name = "device_hfsplus_mav",          .image = NULL},
 	{.name = "device_hfsplus_mav_o",        .image = NULL},
 	{.name = "device_hfsplus_ml",           .image = NULL},
@@ -141,6 +147,8 @@ image_t images[] = {
 
 	{.name = "device_hfsraid",              .image = NULL},
 	{.name = "device_hfsraid_o",            .image = NULL},
+	{.name = "device_hfsraid_yos",          .image = NULL},
+	{.name = "device_hfsraid_yos_o",        .image = NULL},
 	{.name = "device_hfsraid_mav",          .image = NULL},
 	{.name = "device_hfsraid_mav_o",        .image = NULL},
 	{.name = "device_hfsraid_ml",           .image = NULL},
@@ -966,6 +974,12 @@ void drawDeviceIcon(BVRef device, pixmap_t *buffer, position_t p, bool isSelecte
 			{
 				// Use HFS or HFSRAID icon depending on bvr flags. Fallbacks are handled by alt_image above.
 				switch (device->OSVersion[3]) {
+					case '1':
+						if ((device->OSVersion[4]) == '0')
+							devicetype = (device->flags & kBVFlagBooter ? iDeviceHFSRAID_yos : iDeviceHFS_yos);
+						else
+							devicetype = (device->flags & kBVFlagBooter ? iDeviceHFSRAID : iDeviceHFS);
+						break;
 					case '9':
 						devicetype = (device->flags & kBVFlagBooter ? iDeviceHFSRAID_mav : iDeviceHFS_mav);
 						break;
diff --git a/i386/boot2/options.c b/i386/boot2/options.c
index da9dfbb..0089320 100644
--- a/i386/boot2/options.c
+++ b/i386/boot2/options.c
@@ -1201,6 +1201,9 @@ processBootOptions()
 		return -1;
 	}
 
+	// Find out which version mac os we're booting.
+	strncpy(gMacOSVersion, gBootVolume->OSVersion, sizeof(gMacOSVersion));
+
 	// Load config table specified by the user, or use the default.
 
 	if (!getValueForBootKey(cp, "config", &val, &cnt)) {
@@ -1230,10 +1233,14 @@ processBootOptions()
 		{
 			strlcpy( bootInfo->bootFile, val, cnt+1 );
 		} else {
-			strlcpy( bootInfo->bootFile, kDefaultKernel, sizeof(bootInfo->bootFile) );
+			if ((checkOSVersion("10.10"))) {
+				strlcpy( bootInfo->bootFile, kDefaultKernelYosemite, sizeof(bootInfo->bootFile) );
+			} else {
+				strlcpy( bootInfo->bootFile, kDefaultKernel, sizeof(bootInfo->bootFile) );
+			}
 		}
 	}
-	if (strcmp( bootInfo->bootFile, kDefaultKernel ) != 0)
+	if ((strcmp( bootInfo->bootFile, kDefaultKernel ) != 0) && (strcmp( bootInfo->bootFile, kDefaultKernelYosemite ) != 0))
 	{
 		gOverrideKernel = true;
 	}
diff --git a/i386/libsaio/bootstruct.c b/i386/libsaio/bootstruct.c
index ed83594..fd95a79 100644
--- a/i386/libsaio/bootstruct.c
+++ b/i386/libsaio/bootstruct.c
@@ -27,6 +27,7 @@
  */
 
 #include "libsaio.h"
+#include "boot.h"
 #include "bootstruct.h"
 
 #ifndef DEBUG_BOOTSTRUCT
@@ -119,8 +120,7 @@ void initKernBootStruct( void )
 void
 reserveKernBootStruct(void)
 {
-	if ((gMacOSVersion[0] == '1') && (gMacOSVersion[1] == '0')
-		&& (gMacOSVersion[2] == '.') && (gMacOSVersion[3] == '7' || gMacOSVersion[3] == '8' || gMacOSVersion[3] == '9'))
+	if ((checkOSVersion("10.7")) || (checkOSVersion("10.8")) || (checkOSVersion("10.9")) || (checkOSVersion("10.10")))
 	{
 		void *oldAddr = bootArgs;
 		bootArgs = (boot_args *)AllocateKernelMemory(sizeof(boot_args));
